package entregas;

import java.io.Serializable;
import java.util.LinkedHashMap;

import javax.swing.JOptionPane;

/**
 * Las <b>Entregas</b> se agrupan en <b>Colecciones de Entregas</b>.<br>
 * Para agrupar estas colecciones se utiliza la clase <b>LinkedHashMap</b> ya que permite 
 * identificar cada elemento mediante una clave única, el <i>numero de acuse</i> de cada 
 * <b>Entrega</b> y además respeta el orden de inserción de cada elemento.<br>
 * A las <b>Colecciones</b> se les establece un <i>nombre</i> mediante el constructor.
 * Este atributo sirve para representar la <b>Coleccion</b> en pantalla y además se usará 
 * para nombrar el archivo cuando sea guardada en disco.<br>
 * Para que puedan ser guardadas en disco, se implementa la interfaz <b><i>Serializable</i></b>
 * @author Kabuto
 *
 */
@SuppressWarnings("serial")
public final class ColeccionEntregas implements Serializable{
	
	private String nombre;
	private LinkedHashMap<Integer, Entrega> entregas;
	
	public ColeccionEntregas(String nombre) {
		this.nombre = nombre;
		entregas = new LinkedHashMap<Integer, Entrega>();
	}
	
	public String getNombreColeccion() {
		return nombre; 
	}
	
	public Integer getNumeroRegistros() {
		return entregas.size();
	}
	
	/**
	 * Recibe una <b>Entrega</b> para ser insertada en la <b>Coleccion</b><br>
	 * Si ya existe una <b>Entrega</b> con el mismo <i>número de acuse</i>, será rechazada 
	 * y se notificará por pantalla.
	 * @param e <b>Entrega</b> a insertar.
	 * @return <b>true</b> si se pudo insertar, <b>false</b> si fue rechazada.
	 */
	public boolean insertarEntrega(Entrega e) {
		if (entregas.containsKey(e.hashCode()))
		{
			JOptionPane.showMessageDialog(null, "Ya existe una entrega con el acuse: " + e.getNumAcuse(),
					"Insertar Entrega", JOptionPane.WARNING_MESSAGE);
			return false;
		}
		else
		{
			entregas.put(e.hashCode(), e);
			return true;
		}
	}
	
	/**
	 * Recibe una <b>Entrega</b> para eliminar su coincidente en la <b>Coleccion.</b>
	 * @param e <b>Entrega</b> ha ser eliminada.
	 * @return <b>true</b> si el usuario confirma la eliminación, <b>false</b> si el usuario 
	 * decide cancelar la eliminación.
	 */
	public boolean eliminarEntrega(Entrega e) {
		int respuesta = JOptionPane.showConfirmDialog(null, "¿Seguro que quiere eliminar entrega con acuse: " + e.getNumAcuse(),
				"Eliminar Entrega", JOptionPane.YES_NO_OPTION);
		if (respuesta == JOptionPane.YES_OPTION)
		{
			entregas.remove(e.hashCode());
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Recibe una <b>Entrega</b> para sustituir su coincidente en la <b>Coleccion.</b>
	 * @param e <b>Entrega</b> que sustituirá la ya existente.
	 */
	public void modificarEntrega(Entrega e) {
		entregas.put(e.hashCode(), e);
	}
	
	/**
	 * Devuelve la <b>Entrega</b> que ocupa la posición indicada como argumento.<br>
	 * Las posiciones van de <i>1 a n</i>, en lugar del habitual <i>0 a n-1</i>  
	 * @param pos Número de posición que ocupa la <b>Entrega</b> en la <b>Coleccion</b>
	 * @return La <b>Entrega</b> solicitada.<br>Si no existe la posicion indicada se devolverá <i>null</i>
	 */
	public Entrega getEntrega(int pos) {
		if (pos == 0) pos = 1;
		int indice = 1;
		for (Entrega e: entregas.values()) {
			if (indice == pos)
				return e;
			else
				indice++;
		}
		return null;
	}

}
