package entregas;

import java.io.File;

import javax.swing.JOptionPane;

/**
 * Hace de enlace entre la <b>Vista</b> encargada de mostrar información en pantalla
 * y el <b>Modelo</b> encargado de la gestion de datos interna.
 * @author Kabuto
 */
public final class Controlador {
	
	private Vista vista;
	private Modelo modelo;
	
	public void setModelo(Modelo m)
	{
		modelo = m;
	}
	
	public void setVista(Vista v)
	{
		vista = v;
	}
	
	/**
	 * Pregunta al <b>Modelo</b> si los cambios realizados en la
	 * <b>Colección</b> actual han sido guardados en disco o no.
	 * @return <b>true</b> si los cambios están guardados,
	 * <b>false</b> en caso contrario.
	 */
	public boolean cambiosGuardados() {
		return modelo.cambiosGuardados();
	}
	
	/**
	 * Pregunta al <b>Modelo</b> si existen registros en la
	 * <b>Colección</b> actual.
	 * @return <b>true</b> si hay registros, <b>false</b> si
	 * la <b>Colección</b> está vacia. 
	 */
	public boolean hayRegistros() {
		return modelo.hayRegistros();
	}
	
	/**
	 * Pregunta al <b>Modelo</b> la cantidad de registros
	 * de la <b>Colección</b> actual.
	 * @return cantidad de <b>Entregas/<b> registradas.
	 */
	public int getCantidadRegistros() {
		return modelo.getCantidadRegistros();
	}
	
	/**
	 * Pregunta al <b>Modelo</b> la posición del
	 * registro al que apunta actualmente.
	 * @return posición de la <b>Entrega</b> señalada
	 * por el puntero actualmente.
	 */
	public int getRegistroActual() {
		return modelo.getRegistroActual();
	}
	
	/**
	 * Solicita al <b>Modelo</b> que devuelva la <b>Entrega</b>
	 * a la que apunta actualmente.
	 * @return <b>Entrega</b> señalada por el puntero del <b>Modelo</b>
	 */
	public Entrega getEntregaActual() {
		return modelo.getEntregaActual();
	}
	
	/**
	 * Indica al <b>Modelo</b> que cree una nueva <b>Colección</b>
	 * y pide a la <b>Vista</b> que se actualice para esta situación.
	 */
	public void crearNuevaColeccion() {
		if (modelo.crearNuevaColeccion()) 
			vista.cargarDatosNuevaColeccion(modelo.getDatosColeccion());
	}
	
	/**
	 * Solicita al <b>Modelo</b> que devuelva los datos para el marcador
	 * del panel superior, consistentes en en el numero de registros actual 
	 * y la cantidad de registros existentes
	 * @return Array con dos enteros: el registro actual y cantidad de registros existentes.
	 */
	public int[] getDatosParaMarcador() {
		return modelo.datosParaMarcador();
	}
	
	/**
	 * Solicita al <b>Modelo</b> que elimine la <b>Entrega</b> actual
	 * e informe de si tuvo éxito.
	 * @return <b>true</b> si se eliminó la <b>Entrega</b>, <b>false</b> en caso contrario.
	 */
	public boolean eliminarEntregaActual() {
		return modelo.eliminaEntregaActual();
	}
	
	/**
	 * Proceso para guardar una nueva <b>Entrega</b>.<br>
	 * Pide a la <b>Vista</b> que valide los datos del formulario y si son correctos, le pide
	 * que genere una <b>Entrega</b> con esos datos para pasarsela al <b>Modelo</b> y que la 
	 * registre en la <b>Colección</b>.<br>Si la inserción fue correcta, el <b>Modelo</b>
	 * pasará datos a la <b>Vista</b> para que actualice la interfaz con los nuevos cambios.<br>
	 * Si no se pudo insertar por algún motivo, se informará al usuario.
	 */
	public void guardarNuevaEntrega() {
		if (vista.validarFormulario()) {
			if (modelo.addEntrega(vista.devolverEntrega()))
			{
				vista.actualizaTrasGuardarEntrega(modelo.datosParaMarcador());
			}
			else
				JOptionPane.showMessageDialog(null, "No se pudo insertar la nueva Entrega",
						"Error guardando Entrega", JOptionPane.WARNING_MESSAGE);
		}
	}
	
	/**
	 * Proceso para modificar la <b>Entrega</b> actual.<br>
	 * Si la <b>Vista</b> da por válido el formulario, generará una <b>Entrega</b> para pasársela
	 * al <b>Modelo</b>, quien a su vez informará a la <b>Vista</b> de posibles cambios para que 
	 * actualice su interfaz.
	 */
	public void modificarEntrega() {
		if (vista.validarFormulario()) {
			modelo.modificarEntrega(vista.devolverEntrega());
			//Aunque pasamos datos para el marcador, este probablemente no habrá cambiado.
			//Pero la Vista si tendrá que hacer otros cambios, por eso llamamos a este método. 
			vista.actualizaTrasGuardarEntrega(modelo.datosParaMarcador());
		}
	}
	
	/**
	 * Pasa la ruta que ha recibido al <b>Modelo</b> para que guarde en disco
	 * la <b>Colección</b> actual y si ha tenido éxito, pide a la <b>Vista</b> 
	 * que actualice la interfaz a la nueva situación.
	 * @param ruta <b>String</b> con la ruta al directorio donde hacer el guardado.
	 */
	public void guardarColeccion(String ruta) {
		if (modelo.guardarColeccion(ruta))
			vista.actualizaTrasGuardarColeccion();
	}
	
	/**
	 * Pasa el <b>File</b> que ha recibido al <b>Modelo</b> para que abra una 
	 * <b>Colección</b> guardada en disco.<br>Si tiene éxito abriendo el archivo,
	 * el <b>Modelo</b> pasará a la <b>Vista</b> los datos de la <b>Colección</b>
	 * abierta para que actualice la interfaz.
	 * @param f Objeto <b>File</b> que representa el archivo en disco.
	 */
	public void abrirColeccion(File f) {
		if (modelo.abrirColeccion(f))
			vista.cargarDatosNuevaColeccion(modelo.getDatosColeccion());
	}
	
	/**
	 * Pide al <b>Modelo</b> que pase a la siguiente <b>Entrega</b> registrada y que 
	 * informe de si ha sido posible.
	 * @return <b>true</b> si se ha pasado al siguiente registro, <b>false</b> en caso contrario.
	 */
	public boolean siguienteRegistro() {
		return modelo.siguienteRegistro();
	}
	
	/**
	 * Pide al <b>Modelo</b> que pase a la anerior <b>Entrega</b> registrada y que 
	 * informe de si ha sido posible.
	 * @return <b>true</b> si se ha pasado al anterior registro, <b>false</b> en caso contrario.
	 */
	public boolean anteriorRegistro() {
		return modelo.anteriorRegistro();
	}
	
	/**
	 * Este método será invocado desde el <b>JFrame</b> principal cuando se pulse sobre la aspa
	 * de cerrar ventana.<br>Antes de poner fin al programa, se preguntará al <b>Modelo</b> si
	 * los cambios en la <b>Colección</b> han sido guardados. En caso negativo, se avisará al usuario
	 * para que decida si quiere cerrar o no.
	 * @return <b>true</b> si se puede cerrar el programa, <b>false</b> si el usuario no quiere cerrarlo.
	 */
	public boolean avisoAntesDeCerrar(){
		if (modelo.cambiosGuardados()) 
			return true;
		else {
			int opcion = JOptionPane.showConfirmDialog(null, "Hay cambios sin guardar.\n"
					+ "¿Seguro que quiere cerrar programa?\n"
					+ "Los cambios se perderán", "Cerrar Programa", JOptionPane.YES_NO_OPTION);
			if (opcion == JOptionPane.YES_OPTION)
				return true;
			else
				return false;
		}
	}
	
}
