package entregas;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JOptionPane;

/**
 * Consta de una <b>Coleccion de Entregas</b> que será sobre la que estará trabajando el usuario.<br>
 * Se tendrá control sobre si los cambios realizados en la <b>Coleccion</b> han sido guardados en disco o no.<br>
 * Además se controla en todo momento el número de posición de la <b>Entrega</b> que se está visualizando en pantalla 
 * en cada momento.
 * @author Kabuto
 *
 */
public final class Modelo {
	
	ColeccionEntregas entregas;
	boolean cambiosGuardados = true;
	int registroActual = 0;//Puntero que señala la Entrega que está mostrada en pantalla actualmente
	
	public Modelo() {
		
	}
	
	/**
	 * Retorna el valor del puntero de registro que controla la posicion de la <b>Entrega</b> que 
	 * está siendo mostrada en pantalla.
	 * @return Numero de registro o posicion dentro de la <b>Coleccion</b>
	 */
	public int getRegistroActual() {
		return registroActual;
	}
	
	/**
	 * Incrementa la posición del puntero de registro, si es que hay registros posteriores 
	 * en la <b>Colección</b>.
	 * @return <b>true</b> si pudo apuntar al registro siguiente, <b>false</b> si no hay registro siguiente.
	 */
	public boolean siguienteRegistro() {
		if (registroActual < entregas.getNumeroRegistros()) {
			registroActual++;
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Decrementa la posicion del puntero de registro, si es que hay registros anteriores
	 * en la <b>Colección</b>
	 * @return <b>true</b> si pudo apuntar al registro anterior, <b>false</b> si no hay registro anterior.
	 */
	public boolean anteriorRegistro() {
		if (registroActual > 1) {
			registroActual--;
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Informa si los cambios realizados en la <b>Colección</b> han sido 
	 * guardados en disco o no.
	 * @return <b>true</b> si los cambios están guardados, <b>false</b> si hay cambios sin guardar.
	 */
	public boolean cambiosGuardados() {
		return cambiosGuardados;
	}
	
	/**
	 * Devuelve la cantidad de <b>Entregas</b> que hay registradas en 
	 * la <b>Coleccion</b> actual.
	 * @return Cantidad de registros existentes.
	 */
	public int getCantidadRegistros() {
		return entregas.getNumeroRegistros();
	}
	
	/**
	 * Informa si hay registros existentes o si la <b>Coleccion</b> actual 
	 * está vacía.<br>Alternativamente se puede usar el método <i>getNumeroRegistros()</i> 
	 * para saber el número exacto de registros.
	 * @return <b>true</b> si existen registros, <b>false</b> si la <b>Coleccion</b> está vacia.
	 */
	public boolean hayRegistros() {
		return (!(entregas.getNumeroRegistros() == 0));
	}
	
	/**
	 * Devuelve la <b>Entrega</b> a la que apunta el puntero de registros actualmente<br>
	 * Normalmente esta <b>Entrega</b> será la que está siendo mostrada en pantalla.
	 * @return
	 */
	public Entrega getEntregaActual() {
		return entregas.getEntrega(registroActual);
	}
	
	/**
	 * Pide un nombre para poder crear una nueva Coleccion.<br>
	 * Si el usuario cancela, no se creará nada nuevo.
	 * @return <b>true</b> si se creó la nueva coleccion.<br>
	 * <b>false</b> si el usuario canceló y no hay nueva coleccion.
	 */
	public boolean crearNuevaColeccion() {
		String nombre = JOptionPane.showInputDialog(null, "Introduzca un nombre para la Coleccion de Entregas",
				"Nueva Coleccion Entregas", JOptionPane.QUESTION_MESSAGE);
		if (nombre != null) {
			entregas = new ColeccionEntregas(nombre);
			cambiosGuardados = false;
			registroActual = 0;
			return true;
		}
		else return false;
	}
	
	/**
	 * Devuelve un array de tipo <b>Object</b> con el nombre de la coleccion <b>(String)</b>
	 * y el numero de entregas que hay registradas <b>(Integer)</b>.
	 * @return Object[] {String, Integer}
	 */
	public Object[] getDatosColeccion() {
		return new Object[] {entregas.getNombreColeccion(), entregas.getNumeroRegistros()};
	}
	
	/**
	 * Agrega una nueva Entrega a la coleccion.
	 * @param ent <b>Entrega</b> que agregamos
	 * @return <b>true</b> si tuvo exito la insercion.<br>
	 * <b>false</b> si no fue posible.
	 */
	public boolean addEntrega(Entrega ent) {
		if (entregas.insertarEntrega(ent))
		{
			registroActual = entregas.getNumeroRegistros();
			cambiosGuardados = false;
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Recibe una <b>Entrega</b> para sustituir su coincidente dentro de la
	 * <b>Coleccion</b> actual.
	 * @param ent <b>Entrega</b> con los nuevos datos que modificarán la anterior.
	 */
	public void modificarEntrega(Entrega ent) {
		entregas.modificarEntrega(ent);
		cambiosGuardados = false;
	}
	
	/**
	 * Elimina la <b>Entrega</b> señalada por el puntero de registro actual, que 
	 * será la que está siendo mostrada en pantalla.<br>Tras la eliminación, el puntero
	 * pasará a apuntar al registro anterior, si es que quedan registros.
	 * @return
	 */
	public boolean eliminaEntregaActual() {
		if (entregas.eliminarEntrega(getEntregaActual())) {
			cambiosGuardados = false;
			if (hayRegistros())
			{
				if (registroActual > 1)
					registroActual--;
			}
			else
				registroActual = 0;//No hay registros actualmente
			return true;
		}
		else
			return false;
	}
	
	/**
	 * Devuelve un vector de dos enteros con los valores a mostrar en el panel superior
	 * de la interfaz, donde se informa del registro actual y de la cantidad de registros existentes.
	 * @return Array con dos enteros: el registro actual y cantidad de registros existentes.
	 */
	public int[] datosParaMarcador() {
		return new int[]{entregas.getNumeroRegistros(), registroActual};
	}
	
	/**
	 * Guarda la <b>Coleccion</b> actual en un archivo de bytes dentro de la carpeta indicada
	 * en la ruta recibida como parámetro.<br>El nombre del archivo será el <i>nombre</i> de la
	 * <b>Colección</b> y con la extensión "*.entregas"
	 * @param ruta <b>String</b> con la ruta donde se guardará el archivo
	 * @return <b>true</b> si tuvo éxito el guardado, <b>false</b> en caso contrario.
	 */
	public boolean guardarColeccion(String ruta) {
		File fichero = new File(ruta + File.separator + entregas.getNombreColeccion() + ".entregas");
		try {
			ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(fichero));
			oos.writeObject(entregas);
			oos.close();
			cambiosGuardados = true;
			return true;
		} catch (FileNotFoundException e) {
			mostrarAlerta("No se encuentra el fichero/ruta especificado");
			e.printStackTrace();
		} catch (IOException e) {
			mostrarAlerta("Error de acceso al fichero/ruta especificado");
			e.printStackTrace();
		}
		
		return false;
	}
	
	/**
	 * Abre una <b>Coleccion</b> previamente guardada en disco.
	 * @param f Objeto <b>File</b> que representa el archivo que vamos a abrir.
	 * @return <b>true</b> si se cargo con éxito la <b>Coleccion</b>, <b>false</b> en caso contrario.
	 */
	public boolean abrirColeccion(File f) {
		if (f.exists()) {
			try {
				ObjectInputStream ois = new ObjectInputStream(new FileInputStream(f));
				Object aux = ois.readObject();
				if (aux instanceof ColeccionEntregas)
				{
					ois.close();
					entregas = (ColeccionEntregas) aux;
					cambiosGuardados = true;
					registroActual = 1;
					return true;
				}
				else {
					mostrarAlerta("Los datos del fichero no son una Coleccion de Entregas o está corrupto");
					ois.close();
					return false;
				}
			} catch (FileNotFoundException e) {
				mostrarAlerta("No se encuentra el fichero/ruta especificado");
				e.printStackTrace();
			} catch (IOException e) {
				mostrarAlerta("Error de acceso al fichero/ruta especificado");
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		return false;
	}
	
	//Metodos internos
	private void mostrarAlerta(String mensaje) {
		JOptionPane.showMessageDialog(null, mensaje, "Atencion", JOptionPane.WARNING_MESSAGE);
	}

}
