package entregas;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JFileChooser;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileNameExtensionFilter;

import paneles.*;

/**
 * La interfaz está dividida en tres paneles principales:<br><ul>
 * <li>El panel <b>Norte</b> informa sobre la <b>Coleccion</b> actual y proporciona dos botones
 *  para recorrer las <b>Entregas</b> registradas.</li>
 *  <li>El panel <b>Centro</b> muestra todos los campos para los datos de la <b>Entrega</b> visible actualmente.</li>
 *  <li>El panel <b>Sur</b> proporciona botones para gestionar las <b>Colecciones</b> y las <b>Entregas</b></li></ul>
 * @author Kabuto
 *
 */
public final class Vista {

	private Controlador controlador;
	private Container contenedor;
	private Norte panelNorte;
	private Centro panelCentro;
	private Sur panelSur;

	public Vista(Controlador c) {
		controlador = c;
		creaGUI();
	}

	private void creaGUI() {
		try {
			SwingUtilities.invokeAndWait(new Runnable()
			{
				@Override
				public void run()
				{
					panelNorte = new Norte();
					panelNorte.bt_anterior.addActionListener(new AccionAnterior());
					panelNorte.bt_siguiente.addActionListener(new AccionSiguiente());
					panelCentro = new Centro();
					panelSur = new Sur();
					panelSur.bt_nuevaColeccion.addActionListener(new AccionNuevaColeccion());
					panelSur.bt_abrirColeccion.addActionListener(new AccionAbrirColeccion());
					panelSur.bt_guardarColeccion.addActionListener(new AccionGuardarColeccion());
					panelSur.bt_nuevaEntrega.addActionListener(new AccionNuevaEntrega());
					panelSur.bt_modiEntrega.addActionListener(new AccionModificarEntrega());
					panelSur.bt_guardarEntrega.addActionListener(new AccionGuardarEntrega());
					panelSur.bt_eliminarEntrega.addActionListener(new AccionEliminarEntrega());
					contenedor = new Container();
					contenedor.setLayout(new BorderLayout());
					contenedor.add(BorderLayout.NORTH, panelNorte);
					contenedor.add(BorderLayout.CENTER, panelCentro);
					contenedor.add(BorderLayout.SOUTH, panelSur);
				}
			});
		} catch (InvocationTargetException | InterruptedException e) {
			e.printStackTrace();
		}
	}

	public Container getContenedor() {
		return contenedor;
	}

	/**
	 * Recibe los datos sobre la <b>Colección</b> que el panel <b>Norte</b> mostrará.<br>
	 * Estos son un <b>String</b> con el <i>nombre</i> de la <b>Colección</b> y un <b>Integer</b>
	 * con la cantidad de registros que esta contiene.<br>
	 * También decide que botones se activarán en el panel <b>Sur</b> según si la <b>Colección</b>
	 * está vacía o no.<br>
	 * Si no está vacía, el panel <b<Centro</b> mostrará automáticamente la <b<Entrega</b> registrada en la posicion 1.
	 * @param datos Array de <b>Object</b> con dos elementos:<br>
	 * <ul><li><b>String</b> con el nombre a mostrar</li>
	 * <li><b>Integer</b> con la cantidad de registros</li></ul>
	 */
	public void cargarDatosNuevaColeccion(Object[] datos) {
		panelNorte.nuevaColeccion(datos);
		if ((Integer)datos[1] == 0) {//Coleccion esta vacia
			panelSur.activarBotones(false, true, false, false, false);
			panelCentro.resetCampos();
			panelCentro.activarCampos(false);
		}
		else//Coleccion contiene al menos un registro
		{
			panelSur.activarBotones(false, true, true, false, true);
			panelCentro.activarCampos(false);
			panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
		}
	}

	/**
	 * Solicita al panel <b>Centro</b> que revise los campos para validar 
	 * el formulario de datos.<br>Consultar el método <b>Centro</b><i>.validarFormulario()</i>
	 * para ver las reglas de validación.
	 * @return <b>true</b> si el formulario ha sido validado, <b>false</b> en caso contrario.
	 */
	public boolean validarFormulario() {
		return panelCentro.validarFormulario();
	}

	/**
	 * Solicita al panel <b>Centro</b> que recoga los datos del formulario
	 * para generar una <b>Entrega</b>
	 * @return <b>Entrega</b> generada con los datos del formulario.
	 */
	public Entrega devolverEntrega() {
		return panelCentro.devuelveEntrega();
	}

	/**
	 * Actualiza los paneles tras guardar una <b>Entrega</b>.<br>
	 * El panel <b>Norte</b> recibe como argumento los datos para el marcador
	 * solicitados al <b>Modelo.</b><br>El panel <b>Centro</b> desactiva los campos
	 * para que no sean editables hasta que el usuario solicite modificar o crear un
	 * nuevo registro.<br>Al panel <b>Sur</b> se le indica que botones estarán activos
	 * y en que modalidad.
	 * @param datosMarcador Array con dos enteros proporcionados por el <b>Modelo</b>
	 * para que el panel <b>Norte</b> sepa el numero de registro actual y la nueva cantidad
	 * de registros.
	 */
	public void actualizaTrasGuardarEntrega(int[] datosMarcador) {
		panelNorte.actualizaMarcador(datosMarcador);
		panelCentro.activarCampos(false);
		panelSur.activarBotones(true, true, true, false, true);
		panelSur.bt_nuevaEntrega.modo1 = true;//Pasa a modo nueva entrega
		panelSur.bt_nuevaEntrega.setText("<html><p align=\"center\">Nueva<br>Entrega</p></html>");
	}

	/**
	 * INdica al panel <b>Sur</b> que botones estarán activos tras haber
	 * guardado una <b>Colección</b>
	 */
	public void actualizaTrasGuardarColeccion() {
		panelSur.activarBotones(false, true, true, false, true);
	}
	
	//Metodos internos
	/**
	 * Indica al panel <b>Norte</b> que botones estarán activos.<br>
	 * Esto dependerá de si hay registros existentes o no, y de qué
	 * registro se está mostrando actualmente.
	 */
	private void actualizaBotonesNorte() {
		int reg = controlador.getCantidadRegistros();
		if (reg <= 1) {
			panelNorte.bt_anterior.setEnabled(false);
			panelNorte.bt_siguiente.setEnabled(false);
		}
		else//Hay mas de un registro
		{
			int regActual = controlador.getRegistroActual();
			if (regActual == 1) {
				panelNorte.bt_anterior.setEnabled(false);
				panelNorte.bt_siguiente.setEnabled(true);
			}
			else if (regActual == reg) {
				panelNorte.bt_anterior.setEnabled(true);
				panelNorte.bt_siguiente.setEnabled(false);
			}
			else {
				panelNorte.bt_anterior.setEnabled(true);
				panelNorte.bt_siguiente.setEnabled(true);
			}
		}
			
	}

	//Acciones de botones
	/**
	 * Accion para el botón "Nueva Colección".<br>
	 * Pone en marcha el proceso para crear una <b>Colección</b> nueva.
	 * @author Kabuto
	 */
	private class AccionNuevaColeccion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			controlador.crearNuevaColeccion();
			panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
		}
	}

	/**
	 * Accion para el botón "Abrir Coleccion".<br>
	 * Pone en marcha el proceso para abrir una <b>Coleccion</b>
	 * guardad en disco.
	 * @author Kabuto
	 */
	private class AccionAbrirColeccion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fc = new JFileChooser();
			fc.setAcceptAllFileFilterUsed(false);
			FileNameExtensionFilter filter = new FileNameExtensionFilter("Archivos de Entregas", "entregas");
			fc.setFileFilter(filter);
			int seleccion = fc.showOpenDialog(null);
			if (seleccion  == JFileChooser.APPROVE_OPTION) {
				controlador.abrirColeccion(fc.getSelectedFile());
				panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
			}
		}	
	}

	/**
	 * Acción para el botón "Guardar Coleccion".<br>
	 * Pone en marcha el proceso para guardar la <b>Colección</b> actual
	 * en disco.
	 * @author Kabuto
	 */
	private class AccionGuardarColeccion implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			int seleccion = fc.showSaveDialog(null);
			if (seleccion == JFileChooser.APPROVE_OPTION) {
				controlador.guardarColeccion(fc.getSelectedFile().getAbsolutePath());
				panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
			}
		}
	}

	/**
	 * Acción para el botón "Nueva Entrega".<br>
	 * Pone en marcha el proceso para crear una nueva <b>Entrega</b> o bien
	 * cancela dicha creación, ya que este botón tiene dos modalidad de funcionamiento.
	 * @author Kabuto
	 */
	private class AccionNuevaEntrega implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			panelCentro.resetCampos();
			if (panelSur.bt_nuevaEntrega.modo1)
			{
				//Modo nueva entrega
				panelCentro.activarCampos(true);
				panelSur.activarBotones(false, true, false, true, false);
				panelSur.bt_nuevaEntrega.modo1 = false;//Pasa a modo cancelar entrega
				panelSur.bt_nuevaEntrega.setText("<html><p align=\"center\">Cancelar<br>Entrega</p></html>");
			}
			else//Modo cancelar entrega
			{
				panelCentro.activarCampos(false);
				if (controlador.hayRegistros()) {
					panelSur.activarBotones(true, true, true, false, true);
					panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
				}
				else
					panelSur.activarBotones(false, true, false, false, false);
				
				panelSur.bt_nuevaEntrega.modo1 = true;//Pasa a modo nueva entrega
				panelSur.bt_nuevaEntrega.setText("<html><p align=\"center\">Nueva<br>Entrega</p></html>");
			}
		}
	}

	/**
	 * Acción para el botón "Guardar Entrega".<br>
	 * Pone en marcha el proceso para guardar la <b>Entrega</b> actual.<br>
	 * El proceso interno será ligeramente distinto según si la <b>Entrega</b> es nueva o
	 * en realidad es una modificación de una entrega existente.<br>
	 * El botón tiene dos modalidades de funcionamiento para elegir entre un proceso u otro.
	 * @author Kabuto
	 */
	private class AccionGuardarEntrega implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (panelSur.bt_guardarEntrega.modo1) {//Modo normal, añadir una nueva entrega
				controlador.guardarNuevaEntrega();
				panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
			}
			else//MOdo secundario, guardar una modificacion de entrega existente
			{
				controlador.modificarEntrega();
				panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
				//Tras guardar una modificacion, el boton Modificar ha de recuperar el modo primario
				panelSur.bt_modiEntrega.setText("<html><p align=\"center\">Modificar<br>Entrega</p></html>");
				panelSur.bt_modiEntrega.modo1 = true;
			}
				
		}
	}
	
	/**
	 * Acción para el botón "Modificar Entrega".<br>
	 * Pone en marcha el proceso para modificar la <b>Entrega</b> actual o bien
	 * cancela la modificación, ya que este botón tiene dos modalidades de funcionamiento.<br>
	 * Parte del proceso interno consiste en establecer la modalidad del botón "Guardar Entrega" para
	 * que éste sepa que lo que va a guardar es una modificacion y no un registro nuevo. 
	 * @author Kabuto
	 */
	private class AccionModificarEntrega implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (panelSur.bt_modiEntrega.modo1) { //Modo modificar
				panelSur.bt_guardarEntrega.modo1 = false;//Guardar pasa a modo actualizar Entrega en lugar de añadir entrega nueva
				panelCentro.activarCampos(true);
				panelCentro.cam_acuse.setEditable(false);//El campo numero de acuse no se debe modificar
				panelSur.bt_modiEntrega.setText("<html><p align=\"center\">Cancelar<br>Modificar</p></html>");
				panelSur.bt_modiEntrega.modo1 = false;
				panelSur.activarBotones(false, false, true, true, false);
			}
			else {//Modo Cancelar Modificacion
				panelSur.bt_guardarEntrega.modo1 = true;//Guardar vuelver al modo añadir nueva entrega
				panelCentro.activarCampos(false);
				panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
				panelSur.bt_modiEntrega.setText("<html><p align=\"center\">Modificar<br>Entrega</p></html>");
				panelSur.bt_modiEntrega.modo1 = true;
				panelSur.activarBotones(true, true, true, false, true);
			}
		}
	}
	
	/**
	 * Acción para el botón "Eliminar Entrega".<br>
	 * Pone en marcha el proceso para eliminar la <b>Entrega</b> actual.
	 * @author Kabuto
	 */
	private class AccionEliminarEntrega implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (controlador.eliminarEntregaActual())
			{
				panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
				panelNorte.actualizaMarcador(controlador.getDatosParaMarcador());
				actualizaBotonesNorte();
				panelNorte.actualizaAvisoCambios(controlador.cambiosGuardados());
				if (controlador.hayRegistros())
					panelSur.activarBotones(true, true, true, false, true);
				else
					panelSur.activarBotones(true, true, false, false, false);
			}
		}
	}
	
	/**
	 * Acción para el botón "Siguiente".<br>
	 * Pone en marcha el proceso para mostrar la siguiente <b>Entrega</b> registrada.
	 * @author Kabuto
	 */
	private class AccionSiguiente implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (controlador.siguienteRegistro()) {
				panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
				panelNorte.actualizaMarcador(controlador.getDatosParaMarcador());
				actualizaBotonesNorte();
			}
		}
	}
	
	/**
	 * Acción para el botón "Anterior".<br>
	 * Pone en marcha el proceso para mostrar la anterior <b>Entrega</b> registrada.
	 * @author Kabuto
	 */
	private class AccionAnterior implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent arg0) {
			if (controlador.anteriorRegistro()) {
				panelCentro.muestraDatosEntrega(controlador.getEntregaActual());
				panelNorte.actualizaMarcador(controlador.getDatosParaMarcador());
				actualizaBotonesNorte();
			}
		}	
	}

}
